package de.fraunhofer.iem.dataprovider.configuration.security

import de.fraunhofer.iem.dataprovider.configuration.ApiPaths
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.security.config.annotation.web.reactive.EnableWebFluxSecurity
import org.springframework.security.config.web.server.SecurityWebFiltersOrder
import org.springframework.security.config.web.server.ServerHttpSecurity
import org.springframework.security.config.web.server.invoke
import org.springframework.security.core.userdetails.MapReactiveUserDetailsService
import org.springframework.security.core.userdetails.User
import org.springframework.security.core.userdetails.UserDetails
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.security.crypto.password.PasswordEncoder
import org.springframework.security.web.server.SecurityWebFilterChain


const val ADMIN_USERNAME: String = "admin"
const val ADMIN_ROLE: String = "ADMIN"

@Configuration
@EnableWebFluxSecurity
class SecurityConfiguration(val apiKeyFilter: ApiKeyFilter) {

    @Value("\${ADMIN_PASSWORD}")
    lateinit var adminPassword: String

    @Bean
    fun userDetailsService(): MapReactiveUserDetailsService {
        val user: UserDetails = User
            .withUsername(ADMIN_USERNAME)
            .password(passwordEncoder().encode(adminPassword))
            .roles(ADMIN_ROLE)
            .build()
        return MapReactiveUserDetailsService(user)
    }

    /**
     * This filter chain first checks whether this server's api key is present in the
     * request header in the api-key header.
     * Generally, all "read only" endpoints are accessible using the api-key and without
     * further authentication or authorization.
     * All endpoints, which start any calculation are protected with a http basic auth.
     *
     */
    @Bean
    fun springSecurityFilterChain(http: ServerHttpSecurity): SecurityWebFilterChain {
        return http {
            addFilterAt(apiKeyFilter, SecurityWebFiltersOrder.FIRST)
            authorizeExchange {
                authorize(ApiPaths.KPI_ID, permitAll)
                authorize(ApiPaths.KPI_ROOT_REPOSITORY, permitAll)
                authorize(ApiPaths.REPOSITORY, permitAll)
                authorize(ApiPaths.REPOSITORY_ID, permitAll)
                authorize(ApiPaths.GITLAB_REPO_CHANGED, hasRole(ADMIN_ROLE))
                authorize("${ApiPaths.DEBUG}/**", hasRole(ADMIN_ROLE))
                authorize(anyExchange, denyAll)
            }
            httpBasic { }
            csrf {
                disable()
            }
        }
    }

    @Bean
    fun passwordEncoder(): PasswordEncoder {
        return BCryptPasswordEncoder()
    }

}
